Package: kDGLM
Type: Package
Title: Bayesian Analysis of Dynamic Generalized Linear Models
Version: 1.2.14
Authors@R: c(
        person(given="Silvaneo", family ="dos Santos Jr.", email = "silvaneojunior@utexas.edu", role = c("aut", "cre")),
        person(given="Mariane Branco", family ="Alves", email = "mariane@im.ufrj.br", role = c("aut")), 
        person(given="Hélio dos Santos", family="Migon", role = c("aut")))
Description: Provide routines for filtering and smoothing, forecasting, sampling and Bayesian analysis of Dynamic Generalized Linear Models using the methodology described in Alves et al. (2024)<doi:10.48550/arXiv.2201.05387> and dos Santos Jr. et al. (2024)<doi:10.48550/arXiv.2403.13069>.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Imports: extraDistr (>= 1.9.1), Rfast (>= 2.0.8), generics (>= 0.1.3),
        rlang, Rdpack
RdMacros: Rdpack
Suggests: knitr, rmarkdown, ggplot2, plotly, tidyverse, spdep, sf,
        terra, geodata
RoxygenNote: 7.3.3
VignetteBuilder: knitr
URL: https://silvaneojunior.github.io/kDGLM/
Repository: CRAN
BugReports: https://github.com/silvaneojunior/kDGLM/issues
Depends: R (>= 4.1.0)
NeedsCompilation: no
Packaged: 2026-04-07 02:13:34 UTC; Silvaneo
Author: Silvaneo dos Santos Jr. [aut, cre],
  Mariane Branco Alves [aut],
  Hélio dos Santos Migon [aut]
Maintainer: Silvaneo dos Santos Jr. <silvaneojunior@utexas.edu>
Date/Publication: 2026-04-10 20:30:02 UTC
Built: R 4.6.0; ; 2026-04-27 10:24:22 UTC; unix
