# Generated by roxygen2: do not edit by hand

S3method("*",dlm_block)
S3method("+",dlm_block)
S3method(coef,fitted_dlm)
S3method(coefficients,fitted_dlm)
S3method(forecast,fitted_dlm)
S3method(plot,dlm_coef)
S3method(plot,fitted_dlm)
S3method(print,dlm_block)
S3method(print,dlm_distr)
S3method(print,fitted_dlm)
S3method(print,searched_dlm)
S3method(simulate,fitted_dlm)
S3method(specify,dlm_block)
S3method(summary,dlm_block)
S3method(summary,dlm_distr)
S3method(summary,fitted_dlm)
S3method(summary,searched_dlm)
S3method(update,fitted_dlm)
export(AR)
export(CAR_prior)
export(Gamma)
export(Multinom)
export(Normal)
export(Poisson)
export(TF)
export(TF_block)
export(block_mult)
export(block_rename)
export(block_superpos)
export(colQuantile)
export(eval_dlm_log_like)
export(eval_dlm_norm_const)
export(eval_dlm_post)
export(eval_dlm_prior)
export(ffs)
export(ffs_block)
export(fit_model)
export(forecast)
export(gamma_pred)
export(har)
export(harmonic_block)
export(if.na)
export(if.nan)
export(if.null)
export(intervention)
export(joint_prior)
export(kdglm)
export(noise)
export(noise_block)
export(pol)
export(polynomial_block)
export(reg)
export(regression_block)
export(rowQuantile)
export(simulate)
export(smoothing)
export(specify)
export(zero_sum_prior)
import(grDevices)
import(graphics)
importFrom(Rdpack,reprompt)
importFrom(Rfast,Lgamma)
importFrom(Rfast,colAny)
importFrom(Rfast,colCumSums)
importFrom(Rfast,colnth)
importFrom(Rfast,data.frame.to_matrix)
importFrom(Rfast,is.symmetric)
importFrom(Rfast,lower_tri)
importFrom(Rfast,lower_tri.assign)
importFrom(Rfast,matrnorm)
importFrom(Rfast,rowAny)
importFrom(Rfast,rownth)
importFrom(Rfast,transpose)
importFrom(Rfast,upper_tri)
importFrom(Rfast,upper_tri.assign)
importFrom(extraDistr,dbetapr)
importFrom(extraDistr,qbetapr)
importFrom(generics,forecast)
importFrom(generics,specify)
importFrom(grDevices,rainbow)
importFrom(rlang,.data)
importFrom(rlang,is_formula)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,dgamma)
importFrom(stats,dmultinom)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,dt)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.pass)
importFrom(stats,pnorm)
importFrom(stats,qnbinom)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,simulate)
importFrom(stats,update.formula)
importFrom(stats,var)
