## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = FALSE,
  comment = ""
)

## ----eval=FALSE, include=FALSE------------------------------------------------
#  rmarkdown::render("vignettes/intro.Rmd")
#  rmarkdown::render("vignettes/structures.Rmd")
#  rmarkdown::render("vignettes/outcomes.Rmd")
#  rmarkdown::render("vignettes/fitting.Rmd")
#  rmarkdown::render("vignettes/example1.Rmd")

